/* ---------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */

#include "rom400_sock.h"
#include "rom400_init.h"
#include "rom400_task.h"
#include "rom400_xnetstack.h"
#include <stdio.h>
#include <string.h>

/*
 * Sample application showing the use of ping.
 */

// The host to ping
#define HOST_IP_MSB 192
#define HOST_IP_2 168
#define HOST_IP_3 0
#define HOST_IP_LSB 1

// Our IP
#define MY_IP_MSB 192
#define MY_IP_2 168
#define MY_IP_3 0
#define MY_IP_LSB 18

#define MY_SUBNET_MSB 255
#define MY_SUBNET_2 255
#define MY_SUBNET_3 255
#define MY_SUBNET_LSB 0
#define MY_PREFIX 24

#define MY_GATEWAY_MSB 192
#define MY_GATEWAY_2 168
#define MY_GATEWAY_3 0
#define MY_GATEWAY_LSB 1



/*
 * Show our current network configuration.
 */
void network_config()
{
  unsigned char config[60];
  memset(config, 0, 60);

  config[12] = MY_IP_MSB;
  config[13] = MY_IP_2;
  config[14] = MY_IP_3;
  config[15] = MY_IP_LSB;

  config[16] = MY_SUBNET_MSB;
  config[17] = MY_SUBNET_2;
  config[18] = MY_SUBNET_3;
  config[19] = MY_SUBNET_LSB;

  config[20] = MY_PREFIX;

  config[33] = MY_GATEWAY_MSB;
  config[34] = MY_GATEWAY_2;
  config[35] = MY_GATEWAY_3;
  config[36] = MY_GATEWAY_LSB;

  setnetworkparams(config);

  memset(config, 0, 60);
  getnetworkparams(config);

  printf("\nIP %bu.%bu.%bu.%bu/%bu (%bu.%bu.%bu.%bu), gateway %bu.%bu.%bu.%bu.\n\n",
         config[12], config[13], config[14], config[15], config[20],
         config[16], config[17], config[18], config[19],
         config[33], config[34], config[35], config[36]);
}

/*
 * Main entry point.
 */
void main()
{
  struct sockaddr address;
  long rtt, num;
  int error;
  struct pingdata response;
  unsigned char *p, c;



  xnetstack_install();

  printf("DS80C400 PING Demonstration\r\n");
  init_rom(0, 0);
  clear_param_buffers();

  // let's print out a little version information...
  printf("DS80C400 Initialization Library Version %d\r\n", init_version());
  printf("DS80C400 Socket Library Version         %d\r\n", sock_version());
  printf("DS80C400 Scheduler Library Version      %d\r\n", task_version());
  printf("DS80C400 Enhanced Network Stack Version %d\r\n", xnetstack_version());

  network_config();

  // Ping 'em
  memset(&address, 0, sizeof(struct sockaddr));
  address.sin_addr[12] = HOST_IP_MSB;
  address.sin_addr[13] = HOST_IP_2;
  address.sin_addr[14] = HOST_IP_3;
  address.sin_addr[15] = HOST_IP_LSB;

  num = 0;
  while (1) {
    task_sleep(0, 1000);
    memset(response.icmp_data, 0, 32);
    rtt = ping(&address, sizeof(struct sockaddr), 64, &response);
    if (rtt > 0L)
      printf("%ld: %ld ms", num, rtt);
    else if (rtt == 0L)
      printf("%ld: <1 ms", num);
    if (rtt >= 0L) {
      p = response.icmp_data;
      error = 0;
      for (c = 0x20; c < 0x40; c++) {
        if (*p++ != c)
          error = 1;
      }
      if (error) {
        printf(" !!! Data error: ");
        p = response.icmp_data;
        for (c = 0; c < 32; c++)
          printf("%02bx ", *p++);
      }
    }
    else
      printf("%ld: Timeout/Error", num);

    putchar('\n'); 
    num++;
  }

//  puts("\n\nThat's all folks...");

  while(1) 
	 {
	  
	 }
}

